
package kr.as3.component
{
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.text.TextField;
	
	/**
	 * PresentationComponent
	 *
	 * A component that must located at root frame.
	 *
	 * @author: Yun Jinsang (a.k.a. Wooyaggo, http://www.as3.kr, victim4@gmail.com)
	 * @created: 2008.04.15
	 * 
	 * This source code follow LGPL (Lesser GPL)
	 * 
	 
	 
	 * 본 제품(소프트웨어)는 GPL 라이센스 하에 배포되는 소프트웨어 PresentationKit 을 포함합니다
	 
	 
	 * 
	 * 이 소스코드는 LGPL 을 따릅니다.
	 * 사실 저도 LGPL 을 잘 모릅니다.
	 * 
	 * 하지만 단순한 몇가지를 나열해봅니다.
	 * 
	 * * 라이센시(Licensee)는 오픈소스 소프트웨어(이 컴퍼넌트 클래스)를 자유롭게 사용할 수 있다.
	 * * 라이센시(Licensee)는 오픈소스 소프트웨어(이 컴퍼넌트 클래스)를 일정한 조건하에 자유롭게 복제할 수 있다.
	 * * 라이센시(Licensee)는 오픈소스 소프트웨어(이 컴퍼넌트 클래스)를 일정한 조건하에 자유롭게 수정, 수정된 내용을 재배포할 수 있다.
	 * 
	 * * 복제, 수정, 재배포에 대한 조건 *
	 * * 저작권 관련 문구를 유지한다.
	 * * 제품명 혹은 컴퍼넌트(클래스)명을 중복되지 않게 한다.
	 * * 최종결과물에 사용 여부를 명시한다. (논문의 "인용" 과 비슷)
	 * * 수정, 추가하였을때는 수정, 추가한 소스코드도 공개한다. (저작자에게 email 로 알리면 된다고 봄)
	 * * 특허에 관련될 경우 복잡해지니 무시.
	 * 
	 * 
	 * 간단하게 이야기하면 이 소스는 LGPL 을 따르기 때문에 사용했다는 것을 반드시 밝혀야하고, 
	 * 원본 소스가 수정 또는 추가되면 그 소스도 자동으로 LGPL 을 따르게 된다는 이야기입니다.
	 * 
	 * 더 쉽게 말해서 걍 쓰시고 썼다는 이야기만 해주시면 될거 같습니다. 대부분 수정은 잘 안하자나요 ㅋㅋ
	 * 수정하면 수정했다고 공개하거나 메일로 소스랑 같이 알려주면되는거 같구요.
	 * 생각보다 그렇게 어려운 건 아닌거 같습니다^^
	 * 
	 * 오픈소스 소프트웨어 라이센스 가이드 : http://wiki.kldp.org/wiki.php/OpenSourceLicenseGuide?action=show&redirect=FindPage#s-3.2.2
	 * KLDP 의 LGPL 관련 질문글 : http://search.empas.com/search/oea/kldp/oe.html?q=LGPL&sdsn=&en=utf8
	 * 
	 * 우리 모두 같이 자유롭게 소스를 공개하고 떳떳하게 사용하는 분위기를 조성해봅시다.
	 * 
	 */
	public class PresentationComponent extends Sprite
	{
		private var target: MovieClip;
		private var txtPage: TextField;
		
		public function PresentationComponent()
		{
			// root must be MovieClip.
			if( ! this.parent || ! ( this.parent is MovieClip ) )
				throw new Error( "PresentationComponent must be nested and MovieClip." );
			
			this.target = this.parent as MovieClip;
			
			// if txtPage is on root, it will working itself.
			if( this.target.hasOwnProperty( "txtPage" ) && this.target.getChildByName( "txtPage" ) is TextField )
				this.txtPage = this.target.getChildByName( "txtPage" ) as TextField;
			
			this.init();
		}
		
		private function init(): void
		{
			this.target.stop();
			
			var i: int = this.target.totalFrames;
			
			while( i-- )
				this.target.addFrameScript( i, this.onFrameListener );
			
			this.updatePage();
		}
		
		private function onFrameListener(): void
		{
			var childs: Array = this.getChilds();
			
			var n: int = childs.length;
			
			while( n-- )
			{
				// when the scene reached last frame by left click
				PresentationSceneComponent( childs[ n ] ).addEventListener( PresentationSceneEvent.SCENE_COMPLETE, sceneComplete );
				// when the scene reached first frame by right click
				PresentationSceneComponent( childs[ n ] ).addEventListener( PresentationSceneEvent.SCENE_COMPLETE_REVERSE, sceneCompleteReverse );
			}
		}
		
		private function sceneComplete( e: PresentationSceneEvent ): void
		{
			if( this.target.currentFrame == this.target.totalFrames )
				this.target.gotoAndStop( 1 );
			else
				this.target.nextFrame();
			
			this.updatePage();
		}
		
		private function sceneCompleteReverse( e: PresentationSceneEvent ): void
		{
			if( this.target.currentFrame == 1 )
				this.target.gotoAndStop( this.target.totalFrames );
			else
				this.target.prevFrame();
			
			this.updatePage();
		}
		
		private function updatePage(): void
		{
			if( ! this.txtPage )
				return;
			
			this.txtPage.text = this.target.currentFrame + " / " + this.target.totalFrames;
		}
		
		// get childs on current frame.
		private function getChilds(): Array
		{
			var childs: Array = [];
			
			var n: int = this.target.numChildren;
			var i: int;
			var child: MovieClip;
			var childNum: int;
			
			for( i = 0 ; i < n ; i += 1 )
			{
				child = this.target.getChildAt( i ) as MovieClip;
				
				if( child == null )
					continue;
				
				childNum = child.numChildren;
				
				while( childNum-- )
					if( child.getChildAt( childNum ) is PresentationSceneComponent )
					{
						childs.push( child.getChildAt( childNum ) );
						break;
					}
			}
			
			return childs;
		}
	}
}