package kr.as3.component
{
	import flash.display.DisplayObject;
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	import kr.as3.tween.TweenClass;
	import kr.as3.tween.TweenEvent;
	import kr.as3.tween.TweenObject;
	import kr.as3.tween.easing.TweenNone;
	import kr.as3.tween.easing.TweenStrong;
	
	[Event ( name="sceneComplete", type="kr.as3.component.PresentationSceneEvent" ) ]
	[Event ( name="sceneCompleteReverse", type="kr.as3.component.PresentationSceneEvent" ) ]
	
	/**
	 * PresentationSceneComponent
	 *
	 * A component that must located at MovieClip's frame.
	 *
	 * @author: Yun Jinsang (a.k.a. Wooyaggo, http://www.as3.kr, victim4@gmail.com)
	 * @created: 2008.04.15
	 * 
	 * This source code follow LGPL (Lesser GPL)
	 * 
	 
	 
	 * 본 제품(소프트웨어)는 GPL 라이센스 하에 배포되는 소프트웨어 PresentationKit 을 포함합니다
	 
	 
	 * 
	 * 이 소스코드는 LGPL 을 따릅니다.
	 * 사실 저도 LGPL 을 잘 모릅니다.
	 * 
	 * 하지만 단순한 몇가지를 나열해봅니다.
	 * 
	 * * 라이센시(Licensee)는 오픈소스 소프트웨어(이 컴퍼넌트 클래스)를 자유롭게 사용할 수 있다.
	 * * 라이센시(Licensee)는 오픈소스 소프트웨어(이 컴퍼넌트 클래스)를 일정한 조건하에 자유롭게 복제할 수 있다.
	 * * 라이센시(Licensee)는 오픈소스 소프트웨어(이 컴퍼넌트 클래스)를 일정한 조건하에 자유롭게 수정, 수정된 내용을 재배포할 수 있다.
	 * 
	 * * 복제, 수정, 재배포에 대한 조건 *
	 * * 저작권 관련 문구를 유지한다.
	 * * 제품명 혹은 컴퍼넌트(클래스)명을 중복되지 않게 한다.
	 * * 최종결과물에 사용 여부를 명시한다. (논문의 "인용" 과 비슷)
	 * * 수정, 추가하였을때는 수정, 추가한 소스코드도 공개한다. (저작자에게 email 로 알리면 된다고 봄)
	 * * 특허에 관련될 경우 복잡해지니 무시.
	 * 
	 * 
	 * 간단하게 이야기하면 이 소스는 LGPL 을 따르기 때문에 사용했다는 것을 반드시 밝혀야하고, 
	 * 원본 소스가 수정 또는 추가되면 그 소스도 자동으로 LGPL 을 따르게 된다는 이야기입니다.
	 * 
	 * 더 쉽게 말해서 걍 쓰시고 썼다는 이야기만 해주시면 될거 같습니다. 대부분 수정은 잘 안하자나요 ㅋㅋ
	 * 수정하면 수정했다고 공개하거나 메일로 소스랑 같이 알려주면되는거 같구요.
	 * 생각보다 그렇게 어려운 건 아닌거 같습니다^^
	 * 
	 * 오픈소스 소프트웨어 라이센스 가이드 : http://wiki.kldp.org/wiki.php/OpenSourceLicenseGuide?action=show&redirect=FindPage#s-3.2.2
	 * KLDP 의 LGPL 관련 질문글 : http://search.empas.com/search/oea/kldp/oe.html?q=LGPL&sdsn=&en=utf8
	 * 
	 * 우리 모두 같이 자유롭게 소스를 공개하고 떳떳하게 사용하는 분위기를 조성해봅시다.
	 * 
	 */
	public class PresentationSceneComponent extends Sprite
	{
		// tweening time during fade in/out motion, ms
		private const TWEEN_TIME: int = 500;
		// y distance for fade in/out.
		private const TWEEN_AMOUNT: int = 30;
		// delay time between each object's fade in/out motion start.
		private const TWEEN_DELAY: int = 100;
		
		private var target: MovieClip;
		
		public function PresentationSceneComponent()
		{
			if( ! this.parent || ! ( this.parent is MovieClip ) )
				throw new Error( "PresentationSceneComponent must be nested and MovieClip." );
			
			this.target = this.parent as MovieClip;
			this.init();
		}
		
		private function init(): void
		{
			this.target.stop();
			
			// if removed from stage, it will not working by mouse action.
			this.target.addEventListener( Event.ADDED_TO_STAGE, this.addedListener );
			this.target.addEventListener( Event.REMOVED_FROM_STAGE, this.removedListener );
			
			var i: int = this.target.totalFrames;
			
			while( i-- )
				this.target.addFrameScript( i, this.onFrameListener );
			
			this.arrFadedChilds = [];
		}
		
		private function addedListener( e: Event ): void
		{
			this.target.stage.addEventListener( MouseEvent.CLICK, clickListener );
			this.target.stage.addEventListener( MouseEvent.RIGHT_CLICK, rightClickListener );
		}
		
		private function removedListener( e: Event ): void
		{
			this.target.stage.removeEventListener( MouseEvent.CLICK, clickListener );
			this.target.stage.removeEventListener( MouseEvent.RIGHT_CLICK, rightClickListener );
		}
		
		private function clickListener( e: MouseEvent ): void
		{
			if( this.target.currentFrame != this.target.totalFrames )
				this.target.nextFrame();
			else
				this.fadeOut();
		}
		
		private function rightClickListener( e: MouseEvent ): void
		{
			if( this.target.currentFrame != 1 )
				this.target.gotoAndStop( this.target.currentFrame - 1 );
			else
				this.dispatchEvent( new PresentationSceneEvent( PresentationSceneEvent.SCENE_COMPLETE_REVERSE ) );
		}
		
		private function onFrameListener(): void
		{
			this.fadeIn();
		}
		
		private var arrFadeIn: Array;
		private var arrFadedChilds: Array;
		private var fadeInTimer: Timer;
		public function fadeIn(): void
		{
			var childs: Array = this.getChilds();
			
			this.arrFadeIn = [];
			
			var twn: TweenClass;
			var child: DisplayObject;
			
			while( childs.length )
			{
				child = childs.pop() as DisplayObject;
				
				if( this.arrFadedChilds.indexOf( child ) < 0 )
					this.arrFadedChilds.push( child );
				else
					continue;
				
				child.alpha = 0;
				
				twn = new TweenClass();
				twn.target = child;
				twn.time = TWEEN_TIME / 1000;
				twn.tweenObject = new TweenObject();
				twn.tweenObject.y( child.y + TWEEN_AMOUNT, child.y, TweenStrong.easeOut );
				twn.tweenObject.alpha( 0, 1, TweenNone.none );
				
				this.arrFadeIn.push( twn );
			}
			
			if( this.arrFadeIn.length == 0 )
				return;
			
			this.fadeInTimer = new Timer( TWEEN_DELAY, this.arrFadeIn.length );
			this.fadeInTimer.addEventListener( TimerEvent.TIMER, fadeInTimerListener );
			this.fadeInTimer.start();
		}
		
		private function fadeInTimerListener( e: TimerEvent ): void
		{
			var twn: TweenClass = this.arrFadeIn.pop() as TweenClass;
			
			if( twn )
				twn.start();
		}
		
		private var arrFadeOut: Array;
		private var fadeOutTimer: Timer;
		public function fadeOut(): void
		{
			var childs: Array = this.getChilds();
			
			this.arrFadeOut = [];
			
			var twn: TweenClass;
			
			while( childs.length )
			{
				twn = new TweenClass();
				twn.target = childs.pop() as DisplayObject;
				twn.time = TWEEN_TIME / 1500;
				twn.tweenObject = new TweenObject();
				twn.tweenObject.y( twn.target.y, twn.target.y - TWEEN_AMOUNT, TweenStrong.easeIn );
				twn.tweenObject.alpha( twn.target.alpha, 0, TweenNone.none );
				
				this.arrFadeOut.push( twn );
			}
			
			TweenClass( this.arrFadeOut[ 0 ] ).addEventListener( TweenEvent.TWEEN_COMPLETE, this.fadeOutTweenComplete );
			
			this.fadeOutTimer = new Timer( int( TWEEN_DELAY*0.7 ), this.arrFadeOut.length );
			this.fadeOutTimer.addEventListener( TimerEvent.TIMER, fadeOutTimerListener );
			this.fadeOutTimer.start();
		}
		
		private function fadeOutTimerListener( e: TimerEvent ): void
		{
			var twn: TweenClass = this.arrFadeOut.pop() as TweenClass;
			
			if( twn )
				twn.start();
		}
		
		private function fadeOutTweenComplete( e: TweenEvent ): void
		{
			if( this.target.currentFrame == this.target.totalFrames )
				this.dispatchEvent( new PresentationSceneEvent( PresentationSceneEvent.SCENE_COMPLETE ) );
			else
				this.target.nextFrame();
		}
		
		private function getChilds(): Array
		{
			var childs: Array = [];
			
			var n: int = this.target.numChildren;
			var i: int;
			var child: DisplayObject;
			
			for( i = 0 ; i < n ; i += 1 )
			{
				child = this.target.getChildAt( i );
				
				if( ! ( child is PresentationSceneComponent ) )
					childs.push( child );
			}
			
			return childs;
		}
	}
}